@@ -16,10 +16,13 @@ from robosuite.controllers import ALL_CONTROLLERS, load_controller_config
 from robosuite.robots import ALL_ROBOTS
 from robosuite.models.grippers import ALL_GRIPPERS
 
-__version__ = "1.2.1"
+__version__ = "1.3.0"
 __logo__ = """
       ;     /        ,--.
      ["]   ["]  ,<  |__**|
     /[_]\  [~]\/    |//  |
      ] [   OOO      /o|__|
 """
+
+from robosuite.utils.assets_utils import download_assets
+download_assets()
